/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2014 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-Mail: MicroC-8bit@nuvoton.com
//  Date: 27/Jan/2014
//***********************************************************************************************************

//***********************************************************************************************************
//  Application: ADC Function
//  Clock Source from internal RC 20MHz
//  Freq(timer0) =  20M / 12
//  ADC Channel 0 (P0.3) as ADC input
//***********************************************************************************************************

#include "W79E8213.h"
#include "Typedef.h"
#include "Define.h"
#include "Delay.h"
#include <stdio.h>

#define Enabled     0x01

//-----------------------------------------------------------------------------------------------------------
void ADC_ISR(void) interrupt 11                         // ADC interrupt service routine
{
    ADCCON &= CLR_BIT4;                                 // Clear ADC interrupt flag
    ADCCON &= CLR_BIT3;                                 // It is recommended to clear ADCI before ADCS is set
}
//-----------------------------------------------------------------------------------------------------------
void EnableADC(UINT8 u8On)
{
    if (u8On==0)
        AUXR1 &= CLR_BIT2;                              // Disabled ADC circuit
    else
        AUXR1 |= SET_BIT2;                              // Enable ADC circuit

    EADC = Enabled;
    EA = Enabled;
}
//-----------------------------------------------------------------------------------------------------------
void ADCchannel(void)
{
    P0M2 = 0x00;
    P0M1 = 0x08;                                        // P0.3 input only
    PADIDS = 0x08;                                      // Disable digital Input of ADC Input Channel P0.3

    ADCCON1 = ADCCON1 & CLR_BIT2;                       // 0 0 0   ADC0 (P0.3)
    ADCCON  = ADCCON  & CLR_BIT1;
    ADCCON  = ADCCON  & CLR_BIT0;
}
//-----------------------------------------------------------------------------------------------------------
void TriggerADC(void)
{
    ADCCON &= CLR_BIT4;                                 // Clear ADC interrupt flag
    ADCCON |= SET_BIT3;                                 // ADC start
    PCON |= SET_BIT0;                                   // Enter idle mode
}
//-----------------------------------------------------------------------------------------------------------
void main()
{
    ADCchannel();                                       // Select ADC channel 0(P0.3)
    EnableADC(1);

    while(1)
    {
        TriggerADC();
        P1 = ADCH;
        Delay1ms(100);
    }
}








