/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2014 n Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-Mail: MicroC-8bit@nuvoton.com
//  Date: 27/Jan/2014
//***********************************************************************************************************

//***********************************************************************************************************
//  The sample code test all interrupts by setting the individual flag
//  Clock Source from internal RC 20MHz
//  Freq(timer0) =  20M / 12
//***********************************************************************************************************

#include <stdio.h>
#include "W79E8213.h"
#include "Typedef.h"
#include "Define.h"
#include "ISR.h"
#include "Delay.h"

//-----------------------------------------------------------------------------------------------------------
void INT0_ISR()          interrupt 0    // INT0 interrupt service routine ; Vector @  0x03
{
    P0 = 0xFF-0x01;
    clrIE0;
}
//-----------------------------------------------------------------------------------------------------------
void Timer0_ISR()        interrupt 1    // Timer0 interrupt service routine ; Vector @  0x0B
{
    P0 = 0xFF-0x02;
    clrTF0;
}
//-----------------------------------------------------------------------------------------------------------
void INT1_ISR()          interrupt 2    // INT1 interrupt service routine ; Vector @  0x13
{
    P0 = 0xFF-0x03;
    clrIE1;
}
//-----------------------------------------------------------------------------------------------------------
void Timer1_ISR()        interrupt 3    // Timer1 interrupt service routine ; Vector @  0x1B
{
    P0 = 0xFF-0x04;
    clrTF0;
}
//-----------------------------------------------------------------------------------------------------------
void BrownOut_ISR()      interrupt 5    // BrownOut interrupt service routine ; Vector @  0x2B
{
    P0 = 0xFF-0x05;
    clrBOF;
}
//-----------------------------------------------------------------------------------------------------------
void EdgeDetect_ISR()    interrupt 7    // Edge Detect interrupt service routine ; Vector @  0x3B
{
    P0 = 0xFF-0x06;
    clrEDF;
}
//-----------------------------------------------------------------------------------------------------------
void WDT_ISR()           interrupt 10   // WDT interrupt service routine ; Vector @  0x53
{
    P0 = 0xFF-0x07;
    TA = 0xAA;                          // To Enable access WDCON register must first writer in AAH then 55H to TA
    TA = 0x55;
    clrWDIF;
}
//-----------------------------------------------------------------------------------------------------------
void ADC_ISR()           interrupt 11   // ADC interrupt service routine ; Vector @  0x5B
{
    P0 = 0xFF-0x08;
    clrADCI;
}
//-----------------------------------------------------------------------------------------------------------
void PWM_Underflow_ISR() interrupt 14   // PWM underflow interrupt service routine ; Vector @  0x6B
{
    P0 = 0xFF-0x09;
    clrPWMF;
}
//-----------------------------------------------------------------------------------------------------------
void PWM_Brake_ISR()     interrupt 13   // PWM brake interrupt service routine ; Vector @  0x72
{
    P0 = 0xFF-0x0A;
    clrBKF;
}
//-----------------------------------------------------------------------------------------------------------
void LED_Flash(void)
{
    uint8_t u8Cnt;
    for(u8Cnt = 1 ; u8Cnt < 11 ; u8Cnt++)
    {
        P0 = ~P0;
        Delay1ms(500);
    }
}
//-----------------------------------------------------------------------------------------------------------
void All_Interrupt_Initial(void)
{
    IE    = 0x00;               // Disabled all interrupts ; Global, ADC, Brown out,- , Timer1, Ext1, Timer0, Ext0
    EIE   = 0x00;               // Disabled all interrupts ; Edge detect, PWM underflow, PWM brake, WDT

    IP0   = 0x00;               // Set Low priority for all interrupts
    IP0H  = 0x00;
    IP1   = 0x00;
    IP1H  = 0x00;

    TCON  = 0x00;               // Clear all flags ; TR0, TR1 and (IE0, TF0, IE1, TF1)
    clrWDIF;                    // Clear Watch Dog flag
    clrADCI;                    // Clear ADC flag
    EA = 1;                     // Enabled all interrupts
}
//-----------------------------------------------------------------------------------------------------------
void Interrupt_Error(void)
{
    P0 = 0x78;
    while(1);
}
//-----------------------------------------------------------------------------------------------------------
void Ext0_Test(void)
{
    EX0 = 1;                    // Enabled external0 interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    IT0 = 1;                    // Falling Edge Trigger by external input
    setIE0;                     // Set Ext0 Flag to trigger interrupt
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x01)             // Check ISR return value
    {
       Interrupt_Error();
    }
    EX0 = 0;                    // Disabled external0 interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void Timer0_Test(void)
{
    ET0 = 1;                    // Enabled Timer0 interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    setTF0;                     // Set Timer0 Flag to trigger interrupt
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x02)             // Check ISR return value
    {
        Interrupt_Error();
    }
    ET0 = 0;                    // Disabled Timer0 interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void Ext1_Test(void)
{
    EX1 = 1;                    // Enabled external1 interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    IT1 = 1;                    // Falling Edge Trigger by external input
    setIE1;                     // Set Ext1 Flag to trigger interrupt
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x03)             // Check ISR return value
    {
        Interrupt_Error();
    }
    EX1 = 0;                    // Disabled external1 interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void Timer1_Test(void)
{
    ET1 = 1;                    // Enabled Timer1 interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    setTF1;                     // Set Timer1 Flag to trigger interrupt
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x04)             // Check ISR return value
    {
        Interrupt_Error();
    }
    ET1 = 0;                    // Disabled Timer1 interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void Brown_Out_Test(void)
{
    EBO = 1;                    // Enabled Brown out interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    setBOF;
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x05)             // Check ISR return value
        Interrupt_Error();
    EBO = 0;                    // Disabled Brown out interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void Edge_Detect(void)
{
    EED = 1;                    // Enabled Edge Detect interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    setEDF;
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x06)             // Check ISR return value
        Interrupt_Error();
    EED = 0;                    // Disabled Edge Detect interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void WDT_Test(void)
{
    EWDI = 1;                   // Enabled WDT interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    TA = 0xAA;                  // To Enable access WDCON register must first writer in AAH then 55H to TA
    TA = 0x55;
    setWDIF;                    // Set WDT Flag to trigger interrupt
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x07)             // Check ISR return value
        Interrupt_Error();
    EWDI = 0;                   // Disabled WDT interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void ADC_Test(void)
{
    EADC = 1;                   // Enabled ADC interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    setADCI;                    // Set ADC Flag to trigger interrupt
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x08)             // Check ISR return value
        Interrupt_Error();
    EADC = 0;                   // Disabled ADC interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void PWM_Brake_Test(void)
{
    EPW = 1;                    // Enabled PWM Brake interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    setBKF;
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x09)             // Check ISR return value
        Interrupt_Error();
    EPW = 0;                    // Disabled PWM Brake interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void PWM_Underflow_Test(void)
{
    EPWMUF = 1;                 // Enabled PWM underflow interrupt
    P0 = 0xFF;                  // The LED of P1 will be turned off
    setPWMF
    PCON |= SET_BIT0;           // Enter idle mode
    if(P0 != ~0x0A)             // Check ISR return value
        Interrupt_Error();
    EPWMUF = 0;                 // Disabled PWM underflow interrupt
    Delay1ms(500);              // Delay 500ms
}
//-----------------------------------------------------------------------------------------------------------
void main(void)
{
    LED_Flash();                // The LED flash to indicate that the program is running
    All_Interrupt_Initial();
    Ext0_Test();                // External0 interrupt test function
    Timer0_Test();              // Timer0 interrupt test function
    Ext1_Test();                // External1 interrupt test function
    Timer1_Test();              // Timer1 interrupt test function
    Brown_Out_Test();           // Brown Out interrupt test function
    Edge_Detect();              // Edge Detect interrupt test function
    WDT_Test();                 // WDT interrupt test function
    ADC_Test();                 // ADC interrupt test function
    PWM_Brake_Test();           // PWM Brake interrupt test function
    PWM_Underflow_Test();       // PWM Underflow interrupt test function
    while(1);
}








