/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2014 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-Mail: MicroC-8bit@nuvoton.com
//  Date: 27/Jan/2014
//***********************************************************************************************************

//***********************************************************************************************************
//  Application: NVM Function
//  Clock Source from internal RC 20MHz
//***********************************************************************************************************

#include <stdio.h>
#include "W79E8213.h"
#include "Typedef.h"
#include "Define.h"
#include "NVM.h"
#include "Absacc.h"

#define Disabled    0x00

UINT8 mNVMBuffer[16] = {0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F};
//-----------------------------------------------------------------------------------------------------------
void NVM_Page_Erase(UINT8 u8Page_No)
{
    BIT bitTmp;

    bitTmp = EA;
    EA = Disabled;                          // MCU must clear all interrupt

    switch(u8Page_No)
    {
        case 0:
            NVMADDRL = 0x00;                // NVM PAGE0
            break;
        case 1:
            NVMADDRL = 0x10;                // NVM PAGE1
            break;
        case 2:
            NVMADDRL = 0x20;                // NVM PAGE2
            break;
        case 3:
            NVMADDRL = 0x30;                // NVM PAGE3
            break;
        case 4:
            NVMADDRL = 0x40;                // NVM PAGE4
            break;
        case 5:
            NVMADDRL = 0x50;                // NVM PAGE5
            break;
        case 6:
            NVMADDRL = 0x60;                // NVM PAGE6
            break;
        case 7:
            NVMADDRL = 0x70;                // NVM PAGE7
            break;
        default:
            break;
    }//end of switch(u8Page_No)
    setEER;                                 // Set this to erase the page that NVMADDRL was stored
    EA = bitTmp;                            // Restore the EA
}
//-----------------------------------------------------------------------------------------------------------
void NVM_Page_Write(UINT8 u8Page_No)
{
    UINT8 u8CNT, NVMTemp;
    BIT bitTmp;
    bitTmp = EA;
    EA = Disabled;                          // MCU must clear all interrupt

    u8Page_No = u8Page_No*16;

    for (u8CNT = 0; u8CNT<16; u8CNT++)
    {
        NVMADDRL = u8Page_No;               // Write NVM Page Address
        NVMDAT = mNVMBuffer[u8CNT];         // Write NVM Page Data
        setEWR;                             // Trigger the NVM Page Write

        while(1)
        {
            NVMTemp = NVMCON;               // Check ready, EWR=0 if finished
            if((NVMTemp & 0x40) == 0)
            break;
        }//end of while(1)
        u8Page_No++;
    }//end of for (u8CNT =0; u8CNT<16; u8CNT++)
    EA = bitTmp;
}
//-----------------------------------------------------------------------------------------------------------
void NVM_Write_Error(void)
{
    P0 = 0x78;
    while(1);
}
//-----------------------------------------------------------------------------------------------------------
void NVM_Page_Read_Verify(UINT8 u8Page_No)
{
    UINT8 u8CNT,u8Tmp;

    u8Page_No = u8Page_No*16;
    for (u8CNT =0; u8CNT<16; u8CNT++)
    {
        u8Tmp = CBYTE[0xfc00+u8Page_No];    // Read NVM Page Data
        if(u8Tmp != mNVMBuffer[u8CNT])
        {
            NVM_Write_Error();
        }//end of if(u8Tmp != mNVMBuffer[u8CNT])
        u8Page_No ++;
    }//end of for (u8CNT =0; u8CNT<64; u8CNT++)
}
//-----------------------------------------------------------------------------------------------------------
void main(void)
{
    NVM_Page_Erase(1);
    NVM_Page_Write(1);
    NVM_Page_Read_Verify(1);

    P0 = 0x00;
    while(1);
}
//-----------------------------------------------------------------------------------------------------------