/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2014 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-Mail: MicroC-8bit@nuvoton.com
//  Date: 27/Jan/2014
//***********************************************************************************************************

//***********************************************************************************************************
//  Application: Timer Function
//  Clock Source from internal RC 20MHz
//  Freq(timer0) =  20M / 12
//***********************************************************************************************************

#include <stdio.h>
#include "W79E8213.h"
#include "Typedef.h"
#include "Define.h"

#define VALUE_100us     255-167         // 17* (12/20M)  = 100uS
#define VALUE_1ms       65536-1667      // 1667 * (12/20M) = 1mS

#define Disabled        0x00
#define Enabled         0x01

uint32_t u32CNT0 = 0;

//-----------------------------------------------------------------------------------------------------------
void Timer0_ISR(void)   interrupt 1     // Timer0 interrupt service routine
{
    if(!(u32CNT0--))
    {
        P0 = ~(P0 & 0x0F);
        u32CNT0 = 1000;
    }
    TR0 = 0;                            // Stop the Timer1
    TL0 = LOBYTE(VALUE_1ms);
    TH0 = HIBYTE(VALUE_1ms);
    TR0 = 1;                            // Trigger Timer1
}
//-----------------------------------------------------------------------------------------------------------
void Timer0_Initialize(void)
{
    TMOD |= 0x01;                       // Timer0 is 16-bit mode
    TL0 = LOBYTE(VALUE_1ms);
    TH0 = HIBYTE(VALUE_1ms);
    u32CNT0 = 1000;
    ET0 = Enabled;
    EA = Enabled;
    TR0 = 1;                            // Trigger Timer1
}
//-----------------------------------------------------------------------------------------------------------
void Timer1_Initialize(void)
{
    EA = Disabled;
    TMOD |= 0x20;                       // Timer1 is 8-bit auto reload mode
    TH1 = VALUE_100us;
    TL1 = VALUE_100us;
}
//-----------------------------------------------------------------------------------------------------------
void Timer1_Delay100us(uint32_t u32CNT1)
{
    TR1 = 1;
    while (u32CNT1 != 0)                // Trigger Timer1
    {
        while (TF1 != 1);               // Check Timer1 Time-Out Flag
        {
            TF1 = 0;
        }
        u32CNT1 --;
    }
    TR1 = 0;
}
//-----------------------------------------------------------------------------------------------------------
void LED_Flash(void)
{
    P1 = ~(P1 & 0x0F);
    Timer1_Delay100us(20000);           // Delay 2s
}
//-----------------------------------------------------------------------------------------------------------
void main(void)
{
    Timer1_Initialize();
    Timer0_Initialize();

    while(1)
    {
        LED_Flash();
    }
}


