/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2014 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-Mail: MicroC-8bit@nuvoton.com
//  Date: 27/Jan/2014
//***********************************************************************************************************

//***********************************************************************************************************
//  Application: WDT Function
//  Clock Source from internal RC 20MHz
//***********************************************************************************************************

#include <stdio.h>
#include "W79E8213.h"
#include "Typedef.h"
#include "Define.h"
#include "Delay.h"

#define Enabled     0x01
#define Disabled    0x00

//-----------------------------------------------------------------------------------------------------------
void WDT_ISR(void)  interrupt 10
{
    P0 = 0xAA;

    TA = 0xAA;                      // To enable access WDCON register must first writer in AAH then 55H to TA
    TA = 0x55;
    WDCON = 0x01;                   // Reset WDT
}
//-----------------------------------------------------------------------------------------------------------
void LED_Flash(void)
{
    uint8_t u8Cnt;
    for(u8Cnt = 1 ; u8Cnt < 11 ; u8Cnt++)
    {
        P0 = ~P0;
        Delay1ms(500);
    }
}
//-----------------------------------------------------------------------------------------------------------
void WDT_Interrupt_Configure(void)
{
    TA = 0xAA;                      // To enable access WDCON register must first writer in AAH then 55H to TA
    TA = 0x55;
    WDCON |= 0xB0;                  // Only allow to generate WDT interrupt request
    EWDI = Enabled;                 // Enable WDT interrupt
    EA = Enabled;                   // Enable interrupt
}
//-----------------------------------------------------------------------------------------------------------
void WDT_Reset_Configure(void)
{
    EWDI = Disabled;                // Disable WDT interrupt
    EA = Disabled;                  // Disable interrupt
    TA = 0xAA;                      // To enable access WDCON register must first writer in AAH then 55H to TA
    TA = 0x55;
    WDCON |= 0xB2;                  // WDT is running
}
//-----------------------------------------------------------------------------------------------------------
void Error_Condition(void)
{
    P0 = 0x78;

    TA = 0xAA;                      // To enable access WDCON register must first writer in AAH then 55H to TA
    TA = 0x55;                      // Tune of the WDT and clear counter
    WDCON &= 0x01;                  // Stop WDT and reset it
    while(1);
}
//-----------------------------------------------------------------------------------------------------------
void WDT_Check(void)
{
    TA = 0xAA;                      // To enable access WDCON register must first writer in AAH then 55H to TA
    TA = 0x55;                      // Tune of the WDT and clear counter
    WDCON |= 0x01;                  // Stop WDT and reset it's counter
    if(P0 != 0xAA)
    {
        Error_Condition();
    }
}
//-----------------------------------------------------------------------------------------------------------

void main(void)
{
    LED_Flash();                    // The LED flash to indicate that the program is running
    WDT_Interrupt_Configure();      // Enabled WDT interrupt and disabled WDT reset
    Delay1ms(3400);                 // Delay times > (2^26) / 20M = 67.109M / 20M = 3.355s

    WDT_Check();                    // If WDT interrupt service routine was executed,the check function will be pass
    Delay1ms(500);

    WDT_Reset_Configure();          // Disabled WDT interrupt and enable WDT reset
    Delay1ms(3400);                 // Delay times > ((2^26)+512) / 20M = 67.109M / 20M = 3.355s
    Error_Condition();              // LED Display 0x78 that is Error condition. It means WDT doesn't work.
    while(1);
}
